@@ -1,8 +1,8 @@
-# -*- coding: utf-8 -*-
 # Tests for aiohttp/client.py
 
 import gc
 import sys
+from typing import Any
 from unittest import mock
 
 import pytest
@@ -47,6 +47,7 @@ async def test_http_processing_error(session) -> None:
         await response.start(connection)
 
     assert info.value.request_info is request_info
+    response.close()
 
 
 def test_del(session) -> None:
@@ -440,7 +441,11 @@ async def test_text_custom_encoding(loop, session) -> None:
     assert not response.get_encoding.called
 
 
-async def test_text_detect_encoding(loop, session) -> None:
+@pytest.mark.parametrize("content_type", ("text/plain", "text/plain;charset=invalid"))
+async def test_text_charset_resolver(
+    content_type: str, loop: Any, session: Any
+) -> None:
+    session._resolve_charset = lambda r, b: "cp1251"
     response = ClientResponse(
         "get",
         URL("http://def-cl-resp.org"),
@@ -458,7 +463,7 @@ async def test_text_detect_encoding(loop, session) -> None:
         fut.set_result('{"тест": "пройден"}'.encode("cp1251"))
         return fut
 
-    response._headers = {"Content-Type": "text/plain"}
+    response._headers = {"Content-Type": content_type}
     content = response.content = mock.Mock()
     content.read.side_effect = side_effect
 
@@ -466,35 +471,7 @@ async def test_text_detect_encoding(loop, session) -> None:
     res = await response.text()
     assert res == '{"тест": "пройден"}'
     assert response._connection is None
-
-
-async def test_text_detect_encoding_if_invalid_charset(loop, session) -> None:
-    response = ClientResponse(
-        "get",
-        URL("http://def-cl-resp.org"),
-        request_info=mock.Mock(),
-        writer=mock.Mock(),
-        continue100=None,
-        timer=TimerNoop(),
-        traces=[],
-        loop=loop,
-        session=session,
-    )
-
-    def side_effect(*args, **kwargs):
-        fut = loop.create_future()
-        fut.set_result('{"тест": "пройден"}'.encode("cp1251"))
-        return fut
-
-    response._headers = {"Content-Type": "text/plain;charset=invalid"}
-    content = response.content = mock.Mock()
-    content.read.side_effect = side_effect
-
-    await response.read()
-    res = await response.text()
-    assert res == '{"тест": "пройден"}'
-    assert response._connection is None
-    assert response.get_encoding().lower() in ("windows-1251", "maccyrillic")
+    assert response.get_encoding() == "cp1251"
 
 
 async def test_get_encoding_body_none(loop, session) -> None:
@@ -521,7 +498,7 @@ async def test_get_encoding_body_none(loop, session) -> None:
 
     with pytest.raises(
         RuntimeError,
-        match="^Cannot guess the encoding of a not yet read body$",
+        match="^Cannot compute fallback encoding of a not yet read body$",
     ):
         response.get_encoding()
     assert response.closed
@@ -742,9 +719,7 @@ def test_get_encoding_unknown(loop, session) -> None:
     )
 
     response._headers = {"Content-Type": "application/json"}
-    with mock.patch("aiohttp.client_reqrep.chardet") as m_chardet:
-        m_chardet.detect.return_value = {"encoding": None}
-        assert response.get_encoding() == "utf-8"
+    assert response.get_encoding() == "utf-8"
 
 
 def test_raise_for_status_2xx() -> None:
@@ -1257,3 +1232,24 @@ def test_response_links_empty(loop, session) -> None:
     )
     response._headers = CIMultiDict()
     assert response.links == {}
+
+
+def test_response_not_closed_after_get_ok(mocker) -> None:
+    response = ClientResponse(
+        "get",
+        URL("http://del-cl-resp.org"),
+        request_info=mock.Mock(),
+        writer=mock.Mock(),
+        continue100=None,
+        timer=TimerNoop(),
+        traces=[],
+        loop=mock.Mock(),
+        session=mock.Mock(),
+    )
+    response.status = 400
+    response.reason = "Bad Request"
+    response._closed = False
+    spy = mocker.spy(response, "raise_for_status")
+    assert not response.ok
+    assert not response.closed
+    assert spy.call_count == 0
